C PROGRAM SPECTRA *************************************
C TABARE GALLARDO, 2017, (ORIGINAL VERSION 1998), gallardo@fisica.edu.uy
C FACULTAD DE CIENCIAS, URUGUAY
C CALCULATES THE SPECTRUM OF A TIME SERIES, EQUALLY SPACED OR NOT
C MODEL  Y = C1 + C2*COS(2PI*f*T) + C3*SIN(2PI*f*T)
      IMPLICIT REAL*8 (A-H,O-Z)
C UP TO 100000 INPUT DATA, IF YOU NEED MORE CHANGE NTT
      CHARACTER*1 ZC
      PARAMETER (NTT=100000)
      DIMENSION T(NTT),X(NTT),P(NTT),XP(NTT)
      COMMON /VECT/ T,XP,P
      COMMON /VAL/ TWOPI,F1X,N
      TWOPI=DATAN(1.D0)*8.D0
      write(*,*) 'o--------------------------------------------------o'
      write(*,*) '|                      spectra                     |'
      write(*,*) '|                  version 24/01/17                |'
      write(*,*) '|               gallardo@fisica.edu.uy             |'
      write(*,*) 'o--------------------------------------------------o'
      write(*,*) ' '
      write(*,*) 'Spectrum of the time series datos.dat ',
     *'(equally spaced or not)'
      write(*,*) 'fitting to a model y = c1 + c2*cos(2pi*f*t) + ',
     *'c3*sin(2pi*f*t)'
      write(*,*) 'See   Astronomical Journal (1997) vol.113(2), pp 863.'
      write(*,*)
      write(*,*)'The output spectrum is in spectra.dat given by:'
      write(*,*) 'freq, cce, c1, c2, c3, amplitude'
      write(*,*)
      write(*,*) 'Note: unit of frequency = cycles by unit of time'
      write(*,*)
      write(*,*) 'The Spectral Correlation Coefficient, CCE, is related'
      write(*,*)'to the probability that the line is not due to chance.'
      write(*,*)'Look for the maxima of CCE.'
      write(*,*)

      OPEN(1,FILE='datos.dat',STATUS='OLD')
      DO 50 I=1,NTT
        READ(1,*,END=52) T(I),X(I)
  50  CONTINUE
  52  CLOSE(1)
      N=I-1
      write(*,*) 'there are ',n,'  values in the time series'
      ZN=T(N)-T(1)
      WRITE(*,*)
 221     write(*,*)'choose a window: Hanning (h) or square (s)?'
      READ(*,*) ZC
      IF(ZC.EQ.'h'.OR.ZC.EQ.'H') THEN
C LINE WIDTH FOR HANNING WINDOW
      ANCHO=2.D0/ZN
      GOTO 222
      ENDIF
      IF(ZC.EQ.'s'.OR.ZC.EQ.'S') THEN
C LINE WIDTH FOR SQUARE WINDOW  (NO WINDOW)
      ANCHO=1.D0/ZN
      GOTO 222
      ENDIF
      GOTO 221

 222  write(*,*) 'width of each line = ',ancho
      write(*,*)
      write(*,*) 'frequency interval:'
 229     write(*,*) 'initial frequency?'
      read(*,*) frecini
      write(*,*) 'final frequency?'
      read(*,*) frecfin
      if(frecfin.lt.frecini) then
        write(*,*) 'final must be greather than initial...'
        goto 229
      ENDIF
      write(*,*) 'number of points for the spectrum (1000 for example)?'
      READ(*,*) NWF
      NWF=NWF+1
      DNWF1=DFLOAT(NWF-1)
      FFFF=(FRECFIN-FRECINI)/DNWF1
      TAU=T(1)+ZN/2.D0
C DEFINE WEIGHTS CENTERED IN TAU
      SUMAPESO=0.D0
      DO 10 I=1,N
        TTAU=T(I)-TAU
        IF(ZC.EQ.'h'.OR.ZC.EQ.'H') THEN
C HANNING WINDOW
        P(I)=1.D0+DCOS(TWOPI*TTAU/ZN)
        ENDIF
        IF(ZC.EQ.'s'.OR.ZC.EQ.'S') THEN
C SQUARE OR NO WINDOW
        P(I)=1.D0
        ENDIF
        SUMAPESO=SUMAPESO+P(I)
  10  CONTINUE
C NORMALIZING AND DEFINE XP, F1X AND VARIANCE
      F1X=0.D0
      XCUAD=0.D0
      DO 13 I=1,N
        P(I)=P(I)/SUMAPESO
        XP(I)=X(I)*P(I)
        F1X=F1X+XP(I)
        XCUAD=XCUAD+X(I)*XP(I)
  13  CONTINUE
      VARIANZA=XCUAD-F1X*F1X
      write(*,*) 'working....'
      ANCHO100=ANCHO/100.d0
      open(2,file='spectra.dat',status='unknown')
      write(2,*) 'frequency, Spec.Corr.Coeff., C1, C2, C3, amplitude'
       ic=0
      DO 41 NW=1,NWF
        W=FRECINI+FFFF*DFLOAT(NW-1)
C AVOID FREQUENCY EQUAL TO ZERO
          IF(DABS(W).LT.ANCHO100) THEN
             GOTO 41
          END IF
          IC=IC+1
        CALL FOSYL(W,C1,C2,C3,YY)
C amplitude  = DSQRT(C2*C2+C3*C3)
C center = C1
C coefficient of spectral correlation = CCE
        CCE=DABS((YY-F1X*F1X)/VARIANZA)
        AMPLITUD=DSQRT(C2*C2+C3*C3)
        WRITE(2,11) W,CCE,C1,C2,C3,AMPLITUD
  41  CONTINUE
  11  FORMAT(1P6E15.7)
      CLOSE(2)
      STOP
      END

C SUBRUTINE FOSYL ********************************
C FOLLOWING IDEAS FROM GRANT FOSTER AND SYLVIO FERRAZ-MELLO
C MODEL Y = C1 + C2*COS(2PIwt) + C3*SIN(2PIwt)
      SUBROUTINE FOSYL(W,C1,C2,C3,YY)
      IMPLICIT REAL*8 (A-H,O-Z)
C UP TO 100000 INPUT DATA, IF YOU NEED MORE CHANGE NTT
      PARAMETER (NTT=100000)
      DIMENSION XP(NTT),T(NTT),P(NTT)
      DIMENSION CA(NTT),SA(NTT)
      COMMON /VECT/ T,XP,P
      COMMON /VAL/ TWOPI,F1X,N
      F11=1.D0
      F12=0.D0
      F13=0.D0
      F22=0.D0
      F23=0.D0
      F33=0.D0
      F2X=0.D0
      F3X=0.D0
      DO 20 I=1,N
      ALFA=TWOPI*T(I)*W
      AL=DMOD(ALFA,TWOPI)
      CA(I)=DCOS(AL)
      SA(I)=DSIN(AL)
      CAP=CA(I)*P(I)
      SAP=SA(I)*P(I)
      F12=F12+CAP
      F13=F13+SAP
      F22=F22+CA(I)*CAP
      F23=F23+CA(I)*SAP
      F33=F33+SA(I)*SAP
C      F1X=F1X+XP(I)
C F1X IS EQUAL TO THE WEIGHTED VALUE OF X AND Y
      F2X=F2X+XP(I)*CA(I)
      F3X=F3X+XP(I)*SA(I)
   20 CONTINUE
      DET=F11*(F22*F33-F23*F23)
      DET=DET-F12*(F12*F33-F23*F13)
      DET=DET+F13*(F12*F23-F22*F13)
      C1=-F12*(F2X*F33-F23*F3X)+F13*(F2X*F23-F22*F3X)
      C1=(C1+F1X*(F22*F33-F23*F23))/DET
      C2=F11*(F2X*F33-F23*F3X)+F13*(F12*F3X-F2X*F13)
      C2=(C2-F1X*(F12*F33-F23*F13))/DET
      C3=F11*(F22*F3X-F2X*F23)-F12*(F12*F3X-F2X*F13)
      C3=(C3+F1X*(F12*F23-F22*F13))/DET
C CALCULATING YY
      YY=0.D0
      DO 21 I=1,N
      Y=C1+C2*CA(I)+C3*SA(I)
      YY=YY+Y*Y*P(I)
   21 CONTINUE
      RETURN
      END
C *********************************************************
